MQSC commands, throughput, and Perl scripting
JB (jb@lando.net)
MQSeries Specialist (Israel)	c Xephon 2000



> perl mqTP.pl CHANNEL1 5
snd  45.388 KB/sec | rcv       0 KB/sec |     0.2 msg/sec |  226.94 KB/msg
snd   48.63 KB/sec | rcv       0 KB/sec |     0.1 msg/sec |   486.3 KB/msg
snd   48.63 KB/sec | rcv       0 KB/sec |  0.0625 msg/sec |  778.08 KB/msg
snd 61.4911 KB/sec | rcv       0 KB/sec | 0.14285 msg/sec | 430.438 KB/msg
snd 47.8628 KB/sec | rcv       0 KB/sec |     0.4 msg/sec | 119.657 KB/msg
snd 116.104 KB/sec | rcv       0 KB/sec |       2 msg/sec |  58.052 KB/msg
snd 128.124 KB/sec | rcv       0 KB/sec |     1.2 msg/sec |  106.77 KB/msg
snd 126.558 KB/sec | rcv       0 KB/sec |       2 msg/sec | 63.2790 KB/msg
snd 171.397 KB/sec | rcv       0 KB/sec |     0.8 msg/sec | 214.246 KB/msg
snd 175.175 KB/sec | rcv  0.0056 KB/sec |       3 msg/sec | 58.3936 KB/msg
snd 139.182 KB/sec | rcv       0 KB/sec |     0.4 msg/sec | 347.956 KB/msg
snd 140.790 KB/sec | rcv       0 KB/sec |       2 msg/sec | 70.3950 KB/msg
snd 196.703 KB/sec | rcv       0 KB/sec |     0.4 msg/sec | 491.758 KB/msg
snd 142.387 KB/sec | rcv       0 KB/sec |     2.2 msg/sec | 64.7214 KB/msg
snd 162.711 KB/sec | rcv       0 KB/sec |     0.5 msg/sec | 325.422 KB/msg
snd 202.157 KB/sec | rcv       0 KB/sec |     0.2 msg/sec | 1010.78 KB/msg
snd 202.157 KB/sec | rcv       0 KB/sec |     0.1 msg/sec | 2021.57 KB/msg
snd 188.671 KB/sec | rcv       0 KB/sec |     0.2 msg/sec | 943.359 KB/msg
snd 81.8728 KB/sec | rcv       0 KB/sec |     0.2 msg/sec | 409.364 KB/msg
snd 142.515 KB/sec | rcv 0.00466 KB/sec | 2.66666 msg/sec |  53.445 KB/msg







#!/usr/local/bin/perl -w
#---------------------------------------------------------------------
#
# DESCRIPTION:
#
#     This perl script computes MQSeries channel throughput statistics
#     at fixed time intervals. It prints averages for the send rate,
#     receive rate, message rate, and message size during the interval.
#
#     For simplicity, all interaction with MQSeries is made using MQSC
#     commands and not the MQI or any other API set.
#
#     This script can be run on any platform that supports Perl and
#     the runmqsc executable.
#
# USAGE:
#
#     mqTP.pl <chl name> <sampling interval>
#
#     * "chl name" is the name of the channel to sample. This can
#       be a channel of any type.
#
#     * "sampling interval" is the number of seconds between
#       each measurement.
#
#     * The runmqsc executable must be in the path environment
#       variable.
#
#     Example:
#
#       perl mqTP.pl MQ.CHANNEL1 5
#
# INTERNALS:
#
#     * Channel status hash variables: the status of a channel is
#       stored as a hash variable containing the following fields:
#
#       active  Either 1 (channel is active) or 0 (otherwise).
#       now     The time when a sample is taken.
#       msgs    The number of messages sent and received. This
#               corresponds to the MSGS channel status attribute.
#       sent    The total number of bytes sent through the channel.
#       rcvd    The total number of bytes received through the
#               channel.
#
# AUTHOR:
#
#     jb, feb 00.  jb@lando.net
#
#---------------------------------------------------------------------

if ($#ARGV+1 < 2)
{
    &print_usage ();
    die("\n");
}

$chl_name = $ARGV[0];
$sleep_time = $ARGV[1];

%old_chs = &get_chs ($chl_name);
&check_chs (%old_chs);

while (1)
{
    sleep ($sleep_time);

    %new_chs = &get_chs ($chl_name);
    &check_chs (%new_chs);

    print_stats (old_chs, new_chs);

    %old_chs = %new_chs unless $new_chs{"msgs"}-$old_chs{"msgs"} <= 0;
}

# ---------------------------------------------------------------------
# Execute given runmqsc command and return response lines.
#
# TAKES: one-line string containing the mqsc command.
# GIVES: one-line string containing the runmqsc response.
# ---------------------------------------------------------------------
sub runmqsc
{
    my ($line, $lines);

    # This pipes the given mqsc command through runmqsc and
    # grabs the output.
    open (CMDOUT, "echo \"$_[0]\" | runmqsc |") ||
        die("unable to runmqsc given command $_[0]");

    $lines = "";
    while($line=<CMDOUT>)
    {
        $lines .= $line;
    }

    close (CMDOUT);
    return ($lines);
}

# ---------------------------------------------------------------------
# Retrieve channel status measurements.
#
# TAKES: one string containing the name of the channel.
# GIVES: a hash variable containing the channel status measurements.
# ---------------------------------------------------------------------
sub get_chstatus
{
    my (%chstatus, $response);

    %chstatus = ("active" => 0,
                 "msgs" => 0,
                 "rcvd" => 0,
                 "sent" => 0,
                 "now"=> "");

    $chstatus{"now"} = time ();

    $response = &runmqsc("dis chstatus($_[0]) all");

    # Look up phrase "status(running)" in mqsc response.
    # If found, the channel is active.

    if ($response =~ /status\(running\)/i)
    {
        $chstatus {"active"} = 1;
    }
    else
    {
        return (%chstatus);
    }

    # Look up phrases "  msgs(*)", "bytssent(*)", and "bytsrcvd(*)".
    # Note the blank before the phrase "msgs" to differentiate it
    # from the similar expression "curmsgs(*)".

    $response =~ / msgs\((\d*)\)/i;
    $chstatus{"msgs"} = $1;

    $response =~ /bytssent\((\d*)\)/i;
    $chstatus{"sent"} = $1;

    $response =~ /bytsrcvd\((\d*)\)/i;
    $chstatus{"rcvd"} = $1;

    return (%chstatus);
}

# ---------------------------------------------------------------------
# Calculate statistics and print them out.
#
# TAKES: previous and current channel status measurements.
# GIVES: nothing.
# ---------------------------------------------------------------------
sub print_stats
{
    my ($deltat, $deltas,
    $deltar, $deltam, $BSPS, $BRPS, $BPM, $MPS);

    ##%old_chstatus = $_[0];  %new_chstatus = $_[1];
    $deltat = $new_chstatus{"now"}  - $old_chstatus{"now"};
    $deltas = $new_chstatus{"sent"} - $old_chstatus{"sent"};
    $deltar = $new_chstatus{"rcvd"} - $old_chstatus{"rcvd"};
    $deltam = $new_chstatus{"msgs"} - $old_chstatus{"msgs"};

    if ($deltat <= 0)
    {
        die("deltat must be > 0 ! aborting..\n");
    };

    if ($deltam == 0)
    {
        $deltam = 1;
    }

    $BSPS = $deltas / $deltat;             # Bytes sent per second
    $BRPS = $deltar / $deltat;             # Bytes received per second
    $MPS  = $deltam / $deltat;             # Messages per second (S/R)
    $BPM = ($deltas + $deltar)/ $deltam;   # Bytes per message.

    printf ("snd %7.7s KB/sec | ", $BSPS/1000);
    printf ("rcv %7.7s KB/sec | ", $BRPS/1000);
    printf ("%7.7s msg/sec | ",$MPS);
    printf ("%7.7s KB/msg", $BPM/1000);
    printf ("\n");
}

# ---------------------------------------------------------------------
# Print usage summary.
#
# TAKES and GIVES nothing.
# ---------------------------------------------------------------------
sub print_usage
{
    print "USAGE: perl mqTP <chl name> <sampling interval>\n";
}

# ---------------------------------------------------------------------
# Check the status of a channel.
#
# TAKES: the channel status hash variable.
# GIVES: true (channel is active) or false (inactive).
# ---------------------------------------------------------------------
sub check_chstatus
{
    my (%chstatus) = @_;
    if ($chstatus{"active"} == 0)
    {
        die("channel inactive or does not exist, so stopped.\n");
    }
}

